/*
File Name: ISR
 Contents: Code for handling asynchronous service routines for IR detection events
 Last modified: 1/5/11
 Programmer: John Vaughn
 */

/* 
 Routines to check for IR break beam activity go here. Interruption of the IR beam means the corresponding 
 Arduino pin goes HIGH as pull-up resistors are activated on inputs.  Normally IR_0 through IR_3 pins are low 
 when there is no obstruction.  When we find a rising edge, start a timer on that event and end the timer when
 there is a falling edge.  If the timer is beyond a debouncing level, return that device index for logging.
 Currently we are polling for these events as we do not log short duration events and we need more than
 two interrupts to handle all our sensors.
 */

#define DEBOUNCE_TIME 100

// Poll our IR devices and see if we have either rising edges, IRState[i] goes LOW to HIGH, or falling edges.  Other events are ignored.
int updateIR() {
  int currState, event = NOT_FOUND;             // Normal (no obstruction) value is logic LOW
  for (int i = 0; i < IR_SENSORS; i++){         // Loop through all the sensors, compare old and new values
    switch (i) {                                // Get our current state from reading the pin
    case 0: 
      currState = digitalRead(IR_0_PIN); 
      break;
    case 1: 
      currState = digitalRead(IR_1_PIN);       
      break;
    case 2: 
      currState = digitalRead(IR_2_PIN);       
      break;
    case 3: 
      currState = digitalRead(IR_3_PIN);       
      break;
    }  // End switch

    // React to rising edges (IR blocked) by setting up a timer to check for duration of event
    if (IRState[i] < currState) {
      IRTimer[i] = millis(); // Start a timer on this IR channel
    }  // End of rising edge test

    // Falling edges (IR becomes unblocked) mean we just ended a movement event and we record the time elapsed.
    // We check for sensor bounce by requiring a debounce interval be exceeded. 
    if (IRState[i] > currState) {
      IRTimer[i] = millis() - IRTimer[i]; // Got a rising edge, record the time elapsed
      if (IRTimer[i] > DEBOUNCE_TIME) { 
        event = i;
        IRTimer[i] = 1L;
      }  // End if passes debounce test
      else
        IRTimer[i] = 0L;    // Glitch, apparently, do not record as an event
    }  // End if falling edge test

    IRState[i] = currState;  // Make the previous our current
  }  // End for
  return event;
}  // End updateIR

// Utility to clear the channel given by parameter.  Reset the timer to zero to avoid hangovers.
void clearIR(int index) {
  IRTimer[index] = 0L;
}  // End clearIR

// End of file ISR















