/*
File Name: Logging
 Contents: Code for handling format and content of logging.  Here we set the format and content of data to be logged to the
 chosen devices.  We use a longer char string as an accumulation buffer to collect an entire record and then write it to
 the chosen devices.  Much of this is use of strcat C-string function and itoa and lota conversions.
 Last modified: 1/5/11
 Programmer: John Vaughn
 Comments:  There are some odd twists to logging here. We move down through the various sensors, 0-7, and add their data if chosen by the user in the settings 
 menu. Next, since 1-Wire temperature chips have a long access time, we avoid requesting new temperature values for asynchronous events which need to be written
 quickly.  To do this, we reuse previous temp readings for asychronous events when temp readings are selected for logging.
 */
 
// Constants for RTC part indices
#define SECONDS 0
#define MINUTES 1
#define HOURS 2
#define DAYOFWEEK 3
#define DAYOFMONTH 4
#define MONTH 5
#define YEAR 6

// Record formatting stuff
#define COLON ":"
#define SLASH "/"
#define COMMA ","

// Here we assemble data into a buffer from various sources to store in the log file or write to SEEPROM chip.  If preview is enabled, we show the 
// sensor data on the serial display for debugging, ensuring our sensors are reading correctly.  The type parameter tells us if this is an
// a sychronous (regular) or asychronous event to log. 

int logData(int type) {
  char recBuffer[128] = "";          // Holds entire record to write to storage
  char convBuffer[STRING_MAX];      // Holds conversions from int to ASCII, itoa or ltoa
  int rtc[7];                      // Buffer to hold our RTC date time parts

  itoa(itemNum++, convBuffer,10);  // Here goes our item number and event type fields into our buffer.
  strcat(recBuffer,convBuffer);
  strcat(recBuffer,COMMA);         // We only use CSV field demarcations.  Easy to change througout, if desired.
  itoa(type, convBuffer,10); 
  strcat(recBuffer,convBuffer);
  strcat(recBuffer,COMMA);        

  if ( sensorSelected(0) ) {     // Log RTC as sensor 0.  Split into separate time and date fields.
    getDateTime(rtc);
    itoa(rtc[HOURS],convBuffer,10);  // Time format is hh:mn:sec
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,COLON);
    itoa(rtc[MINUTES],convBuffer,10);
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,COLON);
    itoa(rtc[SECONDS],convBuffer,10);
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,COMMA);

    itoa(rtc[DAYOFMONTH],convBuffer,10);  // Date format is DD/MM/YY
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,SLASH);
    itoa(rtc[MONTH],convBuffer,10);
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,SLASH);
    itoa(rtc[YEAR],convBuffer,10);
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,COMMA);
  }  // End log Sensor 0

  if ( sensorSelected(1) ) {     // Log all available temps as sensor 1
    int temp; 
    long unsigned raw;           // Encoded byte code for temp reading.  Get it and decode here into pieces.
    for (int i=0; i < tempAvail; i++) {  // Run through all the temps available
      
      if (type == SYNCH_EVENT) {    // Synchronous logging, wait and get new temp readings
        raw = getTemp(tempAddr[i]); // Get the encoded temp reading
        prevTemp[i] = raw;          // Record it in case we need to substitute it for asynch reading next time
      } 
      else raw = prevTemp[i];        // For asynchronous events, just log previous temps
      
      temp = (int) convertToCelsius(raw);            // Get the whole number part of the reading by truncation of the float
      itoa(temp,convBuffer,10);                      // Convert that int to a string and concat to our buffer
      strcat(recBuffer,convBuffer);
      strcat(recBuffer,".");                        // Add the decimal point
      temp = ((int) (convertToCelsius(raw)*100.0))%100;  // Get the fractional part of the temp, convert it to string and concat to buffer
      itoa(temp,convBuffer,10);
      strcat(recBuffer,convBuffer);
      strcat(recBuffer,COMMA);                        // Always add the CSV COMMA
    }  // End for    
  }  // End if log sensor 1


  if (sensorSelected(2)) {                              // Log IR events as sensor 2
    for (int i = 0; i < IR_SENSORS; i++) {
      if (type == SYNCH_EVENT) ltoa(0L,convBuffer,10); // Don't display confusing asynch data for synch logging
      else 
        ltoa(IRTimer[i],convBuffer,10);                // Put in our data for asynch
      strcat(recBuffer,convBuffer);                    // Convert and concat
      strcat(recBuffer,COMMA);
    }  // End for
  }  // End if

  if (sensorSelected(3)) {                            // Log light level ADC as sensor 3
    itoa(getLightLevel(),convBuffer,10);              // Nothing special, convert and concat
    strcat(recBuffer,convBuffer);
    strcat(recBuffer,COMMA);
  }  // End if

  if (sensorSelected(4)) {                            // Log sensor 4
    strcat(recBuffer,COMMA);
  }

  if (sensorSelected(5)) {                            // Log sensor 5
    strcat(recBuffer,COMMA);
  }

  if (sensorSelected(6)) {                            // Log sensor 6
    strcat(recBuffer,COMMA);
  }

  if (sensorSelected(7)) {                            // Log sensor 7
    strcat(recBuffer,COMMA);
  }

  strcat(recBuffer,"\r\n");          // Add CR and LF to end our record string
  setLogRecordStart(itemNum);        // Put the current record number into Arduino EEPROM 

  if (logSEEPROM) {
    writeSEEPROM(recBuffer);        // Log to SEEPROM, update our Arduino EEPROM pointer now
    setSEEPROM(seepromCurrent);    //  Save to Arduino EERPROM our current SEEPROM address where the next record begins
  }

  if (logSD) {
    if (writeSDStr(recBuffer) != NO_ERROR) Serial.print("SD card error");        // Write to SD card file as chosen
  }

  if (previewSensors) {            // Show the logged data if requested
    Serial.print(recBuffer);
  }
}  // End of logData





















































