/*
File Name: Therm
 Contents: Code for handling DS18B20 family addresses and data.  Based on the Arduino 1-wire library routines.
 Last modified: 1/5/101
 Programmer: John Vaughn
 */

// Pin we read our 1-Wire data and array sizes
#define THERM_PIN 7
#define MAX_THERMS  4
#define ADDR_SIZE   8
#define THERM_FAMILY 0x28

// Interrogate the 1-wire bus and get all the addresses of thermometers reporting.  We
// return the number of such addresses and put them into EEPROM as well.
int getThermAddresses() {

  OneWire tempChip(THERM_PIN);  // Our 1-Wire device 
  byte thermIndex,byteIndex; // Row Index of array entries
  boolean foundAddr;         // Flag to indicate when we find no more addresses or are up to our limit
  byte thermAddr[MAX_THERMS][ADDR_SIZE];      // Array of arrays to hold up to four 8 byte addresses of 1-wire devices

  Serial.println("Getting therms");
  
  // Read and collect the addresses we found
  thermIndex = 0;
  tempChip.reset_search();
  while ( (tempChip.search(thermAddr[thermIndex])) && (thermIndex < MAX_THERMS )) {
    if ( OneWire::crc8( thermAddr[thermIndex], THERM_PIN) != thermAddr[thermIndex][7]) {
      Serial.println("CRC is not valid!\n"); 
      return thermIndex;
    }
    if ( thermAddr[thermIndex][0] != THERM_FAMILY) {
      Serial.println("Device is not a DS18B20 family device.\n"); 
      return thermIndex;
    }
    tempChip.reset();
    tempChip.select(thermAddr[thermIndex]);

    Serial.print("Therm index "); Serial.println(thermIndex,DEC);

    for ( byteIndex = 0; byteIndex < 8; byteIndex++) { 
      // Write our bytes to EEPROM for later logging.  Use THERM_0 as a base and compute others as offsets  
      EEPROM.write(THERM_0 + ADDR_SIZE*thermIndex + byteIndex, thermAddr[thermIndex][byteIndex]); 
      Serial.print(thermAddr[thermIndex][byteIndex],HEX);
    }  // End of printing for 
    Serial.println();
    thermIndex++;
    delay(3000);
  }  // End while
  EEPROM.write(THERMS, thermIndex);
  return thermIndex;  
}  // End of getThermAddresses

// Return the temperature for a temperature chip with the given address in addr[8].  If we 
// get a valid reading we return as an unsigned int so we need to convert before display.
// Temperatures come in two byte parts.

unsigned int getTemp(byte addr[8]){
  OneWire tempChip(THERM_PIN);  // Our 1-Wire device 
  unsigned int result;
  byte ignored;
  byte data[12];
  byte highByte, lowByte;

  tempChip.reset();        // Reset the chip to get ready for reading command
  tempChip.select(addr);  // Float address on bus
  tempChip.write(0x44,1); // Start temp conversion, with parasite power on at the end

  ignored = tempChip.reset();  // Reset to read the temp conversion
  tempChip.select(addr);       // Float the address on the bus
  tempChip.write(0xBE);       // Read Scratchpad command to chip

  for ( int i = 0; i < 9; i++) { // we read 9 bytes of data now
    data[i] = tempChip.read();
  }
  result = (data[1] << 8) + data[0];  // Encode temp into high and low bytes 
  return result;
}  // End of getTemp

//  Convert the temperature in degrees Celsius from a high and low byte encoded in an unsigned int parameter.
float convertToCelsius(unsigned int temp){
  int signBit, scaledTemp, whole, fract;
  signBit = temp & 0x8000;  // test most sig bit
  if (signBit) // Value is negative, handle separately
    temp = (temp ^ 0xffff) + 1;       // 2's comp
  scaledTemp = (6 * temp) + temp / 4; // multiply by (100 * 0.0625) or 6.25
  whole = scaledTemp / 100;  // separate off the whole and fractional portions
  fract = scaledTemp % 100;
  if (signBit) whole *= -1;
  return whole + (fract*0.01);
}  // End of convertToCelsius

// End Therm file

